@@ -1,13 +1,13 @@
 package HTML::Form;
 
-# $Id: Form.pm,v 1.37 2003/10/16 07:25:10 gisle Exp $
+# $Id: Form.pm,v 1.46 2004/06/18 07:13:20 gisle Exp $
 
 use strict;
 use URI;
 use Carp ();
 
 use vars qw($VERSION);
-$VERSION = sprintf("%d.%03d", q$Revision: 1.37 $ =~ /(\d+)\.(\d+)/);
+$VERSION = sprintf("%d.%03d", q$Revision: 1.46 $ =~ /(\d+)\.(\d+)/);
 
 my %form_tags = map {$_ => 1} qw(input textarea button select option);
 
@@ -27,6 +27,8 @@ my %type2class = (
  submit   => "SubmitInput",
  image    => "ImageInput",
  file     => "FileInput",
+
+ keygen   => "KeygenInput",
 );
 
 =head1 NAME
@@ -76,7 +78,7 @@ $response->base() method, as shown by the following example:
 				  $response->base);
 
 The parse() method can parse from an C<HTTP::Response> object
-directly, so the example above can be better written as:
+directly, so the example above can be more conveniently written as:
 
     my $ua = LWP::UserAgent->new;
     my $response = $ua->get("http://www.example.com/form.html");
@@ -94,7 +96,7 @@ sub parse
     my $p = HTML::TokeParser->new(ref($html) ? $html->content_ref : \$html);
     eval {
 	# optimization
-	$p->report_tags(qw(form input textarea select optgroup option));
+	$p->report_tags(qw(form input textarea select optgroup option keygen));
     };
 
     unless (defined $base_uri) {
@@ -127,33 +129,66 @@ sub parse
 		    my $type = delete $attr->{type} || "text";
 		    $attr->{value_name} = $p->get_phrase;
 		    $f->push_input($type, $attr);
-		} elsif ($tag eq "textarea") {
+		}
+		elsif ($tag eq "textarea") {
 		    $attr->{textarea_value} = $attr->{value}
 		        if exists $attr->{value};
 		    my $text = $p->get_text("/textarea");
 		    $attr->{value} = $text;
 		    $f->push_input("textarea", $attr);
-		} elsif ($tag eq "select") {
-		    $attr->{select_value} = $attr->{value}
-		        if exists $attr->{value};
+		}
+		elsif ($tag eq "select") {
+		    # rename attributes reserved to come for the option tag
+		    for ("value", "value_name") {
+			$attr->{"select_$_"} = delete $attr->{$_}
+			    if exists $attr->{$_};
+		    }
 		    while ($t = $p->get_tag) {
 			my $tag = shift @$t;
 			last if $tag eq "/select";
 			next if $tag =~ m,/?optgroup,;
 			next if $tag eq "/option";
 			if ($tag eq "option") {
-			    my %a = (%$attr, %{$t->[0]});
+			    my %a = %{$t->[0]};
+			    # rename keys so they don't clash with %attr
+			    for (keys %a) {
+				next if $_ eq "value";
+				$a{"option_$_"} = delete $a{$_};
+			    }
+			    while (my($k,$v) = each %$attr) {
+				$a{$k} = $v;
+			    }
 			    $a{value_name} = $p->get_trimmed_text;
 			    $a{value} = delete $a{value_name}
 				unless defined $a{value};
 			    $f->push_input("option", \%a);
-			} else {
+			}
+			else {
 			    Carp::carp("Bad <select> tag '$tag'") if $^W;
+			    if ($tag eq "/form" ||
+				$tag eq "input" ||
+				$tag eq "textarea" ||
+				$tag eq "select" ||
+				$tag eq "keygen")
+			    {
+				# MSIE implictly terminate the <select> here, so we
+				# try to do the same.  Actually the MSIE behaviour
+				# appears really strange:  <input> and <textarea>
+				# do implictly close, but not <select>, <keygen> or
+				# </form>.
+				my $type = ($tag =~ s,^/,,) ? "E" : "S";
+				$p->unget_token([$type, $tag, @$t]);
+				last;
+			    }
 			}
 		    }
 		}
+		elsif ($tag eq "keygen") {
+		    $f->push_input("keygen", $attr);
+		}
 	    }
-	} elsif ($form_tags{$tag}) {
+	}
+	elsif ($form_tags{$tag}) {
 	    Carp::carp("<$tag> outside <form>") if $^W;
 	}
     }
@@ -184,10 +219,12 @@ sub push_input
 	Carp::carp("Unknown input type '$type'") if $^W;
 	$class = "TextInput";
     }
-    $class = "IgnoreInput" if exists $attr->{disabled};
     $class = "HTML::Form::$class";
+    my @extra;
+    push(@extra, readonly => 1) if $type eq "hidden";
 
-    my $input = $class->new(type => $type, %$attr);
+    delete $attr->{type}; # don't confuse the type argument
+    my $input = $class->new(type => $type, %$attr, @extra);
     $input->add_to_form($self);
 }
 
@@ -240,7 +277,7 @@ The $name should always be passed in lower case.
 Example:
 
    @f = HTML::Form->parse( $html, $foo );
-   @f = grep $_->attr("id") == "foo", @f;
+   @f = grep $_->attr("id") eq "foo", @f;
    die "No form named 'foo' found" unless @f;
    $foo = shift @f;
 
@@ -515,11 +552,13 @@ sub make_request
 	$uri = URI->new($uri, "http");
 	$uri->query_form(@form);
 	return HTTP::Request->new(GET => $uri);
-    } elsif ($method eq "POST") {
+    }
+    elsif ($method eq "POST") {
 	require HTTP::Request::Common;
 	return HTTP::Request::Common::POST($uri, \@form,
 					   Content_Type => $enctype);
-    } else {
+    }
+    else {
 	Carp::croak("Unknown method '$method'");
     }
 }
@@ -569,6 +608,7 @@ sub click
     for (@{$self->{'inputs'}}) {
         next unless $_->can("click");
         next if $name && $_->name ne $name;
+	next if $_->disabled;
 	return $_->click($self, @_);
     }
     Carp::croak("No clickable input with name $name") if $name;
@@ -705,7 +745,7 @@ error to try to set it to something else and the method will croak if
 you try.
 
 You will also be able to set the value of read-only inputs, but a
-warning will be generated if running under 'perl -w'.
+warning will be generated if running under C<perl -w>.
 
 =cut
 
@@ -763,6 +803,41 @@ sub value_names {
     return
 }
 
+=item $bool = $input->readonly
+
+=item $input->readonly( $bool )
+
+This method is used to get/set the value of the readonly attribute.
+You are allowed to modify the value of readonly inputs, but setting
+the value will generate some noise when warnings are enabled.  Hidden
+fields always start out readonly.
+
+=cut
+
+sub readonly {
+    my $self = shift;
+    my $old = $self->{readonly};
+    $self->{readonly} = shift if @_;
+    $old;
+}
+
+=item $bool = $input->disabled
+
+=item $input->disabled( $bool )
+
+This method is used to get/set the value of the disabled attribute.
+Disabled inputs do not contribute any key/value pairs for the form
+value.
+
+=cut
+
+sub disabled {
+    my $self = shift;
+    my $old = $self->{disabled};
+    $self->{disabled} = shift if @_;
+    $old;
+}
+
 =item $input->form_name_value
 
 Returns a (possible empty) list of key/value pairs that should be
@@ -775,6 +850,7 @@ sub form_name_value
     my $self = shift;
     my $name = $self->{'name'};
     return unless defined $name;
+    return if $self->{disabled};
     my $value = $self->value;
     return unless defined $value;
     return ($name => $value);
@@ -790,25 +866,31 @@ sub dump
     my $dump = "$name=$value";
 
     my $type = $self->type;
-    return $dump if $type eq "text";
 
-    $type = ($type eq "text") ? "" : " ($type)";
-    my $menu = $self->{menu} || "";
-    my $value_names = $self->{value_names};
-    if ($menu) {
-	my @menu;
-	for (0 .. @$menu-1) {
-	    my $opt = $menu->[$_];
-	    $opt = "<UNDEF>" unless defined $opt;
-	    substr($opt,0,0) = "*" if $self->{seen}[$_];
-	    $opt .= "/$value_names->[$_]"
-		if $value_names && defined $value_names->[$_]
-		    && $value_names->[$_] ne $opt;
-	    push(@menu, $opt);
+    $type .= " disabled" if $self->disabled;
+    $type .= " readonly" if $self->readonly;
+    return sprintf "%-30s %s", $dump, "($type)" unless $self->{menu};
+
+    my @menu;
+    my $i = 0;
+    for (@{$self->{menu}}) {
+	my $opt = $_->{value};
+	$opt = "<UNDEF>" unless defined $opt;
+	$opt .= "/$_->{name}"
+	    if defined $_->{name} && length $_->{name} && $_->{name} ne $opt;
+	substr($opt,0,0) = "-" if $_->{disabled};
+	if (exists $self->{current} && $self->{current} == $i) {
+	    substr($opt,0,0) = "!" unless $_->{seen};
+	    substr($opt,0,0) = "*";
 	}
-	$menu = "[" . join("|", @menu) . "]";
+	else {
+	    substr($opt,0,0) = ":" if $_->{seen};
+	}
+	push(@menu, $opt);
+	$i++;
     }
-    sprintf "%-30s %-10s %s", $dump, $type, $menu;
+
+    return sprintf "%-30s %-10s %s", $dump, "($type)", "[" . join("|", @menu) . "]";
 }
 
 
@@ -827,9 +909,8 @@ sub value
     my $old = $self->{value};
     $old = "" unless defined $old;
     if (@_) {
-	if (exists($self->{readonly}) || $self->{type} eq "hidden") {
-	    Carp::carp("Input '$self->{name}' is readonly") if $^W;
-	}
+        Carp::carp("Input '$self->{name}' is readonly")
+	    if $^W && $self->{readonly};
 	$self->{value} = shift;
     }
     $old;
@@ -861,24 +942,31 @@ sub new
 
     my $value = delete $self->{value};
     my $value_name = delete $self->{value_name};
-    
-    if ($self->type eq "checkbox") {
+    my $type = $self->{type};
+
+    if ($type eq "checkbox") {
 	$value = "on" unless defined $value;
-	$self->{menu} = [undef, $value];
-	$self->{value_names} = ["off", $value_name];
-	$self->{current} = (exists $self->{checked}) ? 1 : 0;
+	$self->{menu} = [
+	    { value => undef, name => "off", },
+            { value => $value, name => $value_name, },
+        ];
+	$self->{current} = (delete $self->{checked}) ? 1 : 0;
+	;
+    }
+    else {
+	$self->{option_disabled}++
+	    if $type eq "radio" && delete $self->{disabled};
+	$self->{menu} = [
+            {value => $value, name => $value_name},
+        ];
+	my $checked = $self->{checked} || $self->{option_selected};
 	delete $self->{checked};
-    } else {
-	$self->{menu} = [$value];
-	my $checked = exists $self->{checked} || exists $self->{selected};
-	delete $self->{checked};
-	delete $self->{selected};
+	delete $self->{option_selected};
 	if (exists $self->{multiple}) {
-	    unshift(@{$self->{menu}}, undef);
-	    $self->{value_names} = ["off", $value_name];
+	    unshift(@{$self->{menu}}, { value => undef, name => "off"});
 	    $self->{current} = $checked ? 1 : 0;
-	} else {
-	    $self->{value_names} = [$value_name];
+	}
+	else {
 	    $self->{current} = 0 if $checked;
 	}
     }
@@ -889,17 +977,25 @@ sub add_to_form
 {
     my($self, $form) = @_;
     my $type = $self->type;
+
     return $self->SUPER::add_to_form($form)
-	if $type eq "checkbox" ||
-	   ($type eq "option" && exists $self->{multiple});
+	if $type eq "checkbox";
+
+    if ($type eq "option" && $self->{multiple}) {
+	$self->{disabled} ||= $self->{option_disabled};
+	return $self->SUPER::add_to_form($form);
+    }
+
+    die "Assert" if @{$self->{menu}} != 1;
+    my $m = $self->{menu}[0];
+    $m->{disabled}++ if $self->{option_disabled};
 
     my $prev = $form->find_input($self->{name}, $self->{type});
     return $self->SUPER::add_to_form($form) unless $prev;
 
     # merge menues
-    push(@{$prev->{menu}}, @{$self->{menu}});
-    push(@{$prev->{value_names}}, @{$self->{value_names}});
-    $prev->{current} = @{$prev->{menu}} - 1 if exists $self->{current};
+    $prev->{current} = @{$prev->{menu}} if exists $self->{current};
+    push(@{$prev->{menu}}, $m);
 }
 
 sub fixup
@@ -908,42 +1004,44 @@ sub fixup
     if ($self->{type} eq "option" && !(exists $self->{current})) {
 	$self->{current} = 0;
     }
-    $self->{seen} = [(0) x @{$self->{menu}}];
-    $self->{seen}[$self->{current}] = 1 if exists $self->{current};
+    $self->{menu}[$self->{current}]{seen}++ if exists $self->{current};
 }
 
 sub value
 {
     my $self = shift;
     my $old;
-    $old = $self->{menu}[$self->{current}] if exists $self->{current};
+    $old = $self->{menu}[$self->{current}]{value} if exists $self->{current};
     if (@_) {
 	my $i = 0;
 	my $val = shift;
 	my $cur;
+	my $disabled;
 	for (@{$self->{menu}}) {
-	    if ((defined($val) && defined($_) && $val eq $_) ||
-		(!defined($val) && !defined($_))
+	    if ((defined($val) && defined($_->{value}) && $val eq $_->{value}) ||
+		(!defined($val) && !defined($_->{value}))
 	       )
 	    {
 		$cur = $i;
-		last;
+		$disabled = $_->{disabled};
+		last unless $disabled;
 	    }
 	    $i++;
 	}
-	unless (defined $cur) {
+	if (!(defined $cur) || $disabled) {
 	    if (defined $val) {
 		# try to search among the alternative names as well
 		my $i = 0;
 		my $cur_ignorecase;
 		my $lc_val = lc($val);
-		for (@{$self->{value_names}}) {
-		    if (defined $_) {
-			if ($val eq $_) {
+		for (@{$self->{menu}}) {
+		    if (defined $_->{name}) {
+			if ($val eq $_->{name}) {
+			    $disabled = $_->{disabled};
 			    $cur = $i;
-			    last;
+			    last unless $disabled;
 			}
-			if (!defined($cur_ignorecase) && $lc_val eq lc($_)) {
+			if (!defined($cur_ignorecase) && $lc_val eq lc($_->{name})) {
 			    $cur_ignorecase = $i;
 			}
 		    }
@@ -951,7 +1049,10 @@ sub value
 		}
 		unless (defined $cur) {
 		    $cur = $cur_ignorecase;
-		    unless (defined $cur) {
+		    if (defined $cur) {
+			$disabled = $self->{menu}[$cur]{disabled};
+		    }
+		    else {
 			my $n = $self->name;
 		        Carp::croak("Illegal value '$val' for field '$n'");
 		    }
@@ -962,8 +1063,12 @@ sub value
 	        Carp::croak("The '$n' field can't be unchecked");
 	    }
 	}
+	if ($disabled) {
+	    my $n = $self->name;
+	    Carp::croak("The value '$val' has been disabled for field '$n'");
+	}
 	$self->{current} = $cur;
-	$self->{seen}[$cur] = 1;
+	$self->{menu}[$cur]{seen}++;
     }
     $old;
 }
@@ -989,29 +1094,27 @@ sub check
 {
     my $self = shift;
     $self->{current} = 1;
-    $self->{seen}[1] = 1;
+    $self->{menu}[1]{seen}++;
 }
 
 sub possible_values
 {
     my $self = shift;
-    @{$self->{menu}};
+    map $_->{value}, @{$self->{menu}};
 }
 
 sub other_possible_values
 {
     my $self = shift;
-    map { $self->{menu}[$_] }
-        grep {!$self->{seen}[$_]}
-             0 .. (@{$self->{seen}} - 1);
+    map $_->{value}, grep !$_->{seen}, @{$self->{menu}};
 }
 
 sub value_names {
     my $self = shift;
     my @names;
-    for my $i (0 .. @{$self->{menu}} - 1) {
-	my $n = $self->{value_names}[$i];
-	$n = $self->{menu}[$i] unless defined $n;
+    for (@{$self->{menu}}) {
+	my $n = $_->{name};
+	$n = $_->{value} unless defined $n;
 	push(@names, $n);
     }
     @names;
@@ -1058,10 +1161,11 @@ sub form_name_value
     my $self = shift;
     my $clicked = $self->{clicked};
     return unless $clicked;
+    return if $self->{disabled};
     my $name = $self->{name};
-    return unless defined $name;
-    return ("$name.x" => $clicked->[0],
-	    "$name.y" => $clicked->[1]
+    $name = (defined($name) && length($name)) ? "$name." : "";
+    return ("${name}x" => $clicked->[0],
+	    "${name}y" => $clicked->[1]
 	   );
 }
 
@@ -1146,6 +1250,7 @@ sub form_name_value {
 
     my $name = $self->name;
     return unless defined $name;
+    return if $self->{disabled};
 
     my $file = $self->file;
     my $filename = $self->filename;
@@ -1172,6 +1277,19 @@ sub form_name_value {
     return ($name => [$file, $filename, @headers]);
 }
 
+package HTML::Form::KeygenInput;
+@HTML::Form::KeygenInput::ISA=qw(HTML::Form::Input);
+
+sub challenge {
+    my $self = shift;
+    return $self->{challenge};
+}
+
+sub keytype {
+    my $self = shift;
+    return lc($self->{keytype} || 'rsa');
+}
+
 1;
 
 __END__
